use std::sync::Arc;

use actix::prelude::*;
use libmcaptcha::{
    cache::{hashcache::HashCache, messages::VerifyCaptchaResult},
    master::embedded::master::Master,
    master::messages::AddSiteBuilder,
    pow::{ConfigBuilder, Work},
    system::{System, SystemBuilder},
    DefenseBuilder, LevelBuilder, MCaptchaBuilder,
};

pub fn init_system(salt: String) -> Arc<System<HashCache, Master>> {
    let cache = HashCache::default().start();

    let pow = ConfigBuilder::default().salt(salt).build().unwrap();

    let master = Master::new(5).start();

    let system = SystemBuilder::default()
        .master(master)
        .cache(cache)
        .pow(pow.clone())
        .runners(4)
        .queue_length(2000)
        .build();
    Arc::new(system)
}
